@@ -27,4 +27,8 @@ SEXP lmer_variances(SEXP x);
 SEXP lmer_Crosstab(SEXP flist);
 SEXP lmer_firstDer(SEXP x, SEXP val);
 
+/*   EXPERIMENTAL!   EXPERIMENTAL!   EXPERIMENTAL!  */
+SEXP lmer_collapse(SEXP x);
+SEXP lmer_laplace_devComp(SEXP x);
+
 #endif
